###################################################################################################
# _metadictmanager.py
#
# Implementation of class _metadictmanager.MetadictManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import AccessControl.Role
import string
import urllib
import copy


# -------------------------------------------------------------------------------------------------
#  getAttrName
# -------------------------------------------------------------------------------------------------
def getAttrName(id):
  s_attr = id
  s_attr = string.lower(s_attr)
  s_attr = string.replace(s_attr,'.','_')
  s_attr = 'attr_%s'%s_attr
  return s_attr

# -------------------------------------------------------------------------------------------------
#  changeMetaDictAttrs
# -------------------------------------------------------------------------------------------------
def changeMetaDictAttrs(self, old_attr, new_attr):
  if self.meta_type[:3] == 'ZMS':
    for s_preview in [None,'preview']:
      req = {'preview':s_preview}
      ob = self.getObjVersion(req)
      for s_lang in self.getLanguages():
        old_lang_attr = "%s_%s"%(old_attr,s_lang)
        new_lang_attr = "%s_%s"%(new_attr,s_lang)
        if hasattr(ob,old_lang_attr):
          try:
            setattr(ob,new_lang_attr,copy.deepcopy(getattr(ob,old_lang_attr)))
          except:
            pass
    for ob in self.objectValues():
      changeMetaDictAttrs(ob,old_attr,new_attr)

# -------------------------------------------------------------------------------------------------
#  deleteMetaDictAttrs
# -------------------------------------------------------------------------------------------------
def deleteMetaDictAttrs(self, attr):
  if self.meta_type[:3] == 'ZMS':
    for s_lang in self.getLanguages():
      lang_attr = "%s_%s"%(attr,s_lang)
      if hasattr(self,lang_attr):
        try:
          delattr(self,lang_attr)
        except:
          pass
    for ob in self.objectValues():
      deleteMetaDictAttrs(ob,attr)

# -------------------------------------------------------------------------------------------------
#  initMetaDictAttrs
# -------------------------------------------------------------------------------------------------
def initMetaDictAttrs(self):
  s_lang = self.getPrimaryLanguage()
  dst_meta_types = ['ZMS','ZMSRubrik','ZMSDocument']
  if not "attr_dc_description" in self.getMetadictAttrs():
    self.__set_metadict__(None,"DC.Description",self.getLangStr("ATTR_DC_DESCRIPTION",s_lang),'textarea','',dst_meta_types)
  if not "attr_dc_subject" in self.getMetadictAttrs():
    self.__set_metadict__(None,"DC.Subject",self.getLangStr("ATTR_DC_SUBJECT",s_lang),'textarea','',dst_meta_types)
  if not "attr_dc_type" in self.getMetadictAttrs():
    self.__set_metadict__(None,"DC.Type",self.getLangStr("ATTR_DC_TYPE",s_lang),'select',self.getLangStr("OPT_ATTR_DC_TYPE",s_lang),dst_meta_types)
  if not "attr_dc_creator" in self.getMetadictAttrs():
    self.__set_metadict__(None,"DC.Creator",self.getLangStr("ATTR_DC_CREATOR",s_lang),'input','',dst_meta_types)
  if not "attr_channel" in self.getMetadictAttrs():
    self.__set_metadict__(None,"Channel",self.getLangStr("ATTR_CHANNEL",s_lang),'select',self.getLangStr("OPT_ATTR_CHANNEL",s_lang),dst_meta_types)
  if not "attr_robots" in self.getMetadictAttrs():
    self.__set_metadict__(None,"Robots",self.getLangStr("ATTR_ROBOTS",s_lang),'select','all\nnone\nindex,nofollow\nnoindex/follow',dst_meta_types)


###################################################################################################
###################################################################################################
###
###   class MetadictManager
###
###################################################################################################
###################################################################################################
class MetadictManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
                'manage_changeMetadicts','manage_customizeMetadictForm',
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
                ('ZMS Author', __authorPermissions__),
                ('ZMS Editor', __editorPermissions__),
                )
                

    # Management Interface to customize DC.Metadictionaries.
    manage_customizeMetadictForm = HTMLFile('dtml/metadict/manage_customizeform', globals()) 

    # ------------------------------------------------------------------------------------------
    # MetadictManager.recurse_updateVersionMetaDicts: 
    #
    # Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersionMetaDicts(self, lang, manage_lang, REQUEST):
      message = ""
      old_metadicts = self.__get_metadicts__()
      
      # ---------------------------------------------------------------------
      # Convert dictionary to list.
      # ---------------------------------------------------------------------
      if type(old_metadicts) is type({}):
        new_metadicts = []
        for id in old_metadicts.keys():
          ob = copy.deepcopy(old_metadicts[id])
          multiple = ob['multiple']
          keys = ob['keys']
          ob['id'] = id
          if multiple == 1 and len(keys) > 0:
            ob['datatype'] = 'LangList'
          else:
            ob['datatype'] = 'LangString'
          if len(keys) == 0:
            if multiple == 1:
              ob['type'] = 'textarea'
            else:
              ob['type'] = 'input'
          else:
            if multiple == 1:
              ob['type'] = 'multiselect'
            else:
              ob['type'] = 'select'
          new_metadicts.append(getAttrName(id))
          new_metadicts.append(ob)
        self.__attr_dc_metadicts__ = copy.deepcopy(new_metadicts)

      # ---------------------------------------------------------------------
      # 26.03.2002
      # Build #069: Remove multiple.
      # ---------------------------------------------------------------------
      if getattr(self,'build','') < '069':
        old_metadicts = self.__attr_dc_metadicts__
        new_metadicts = []
        for old_metadict in old_metadicts:
          if type(old_metadict) is type({}):
            new_metadict = {}
            for key in old_metadict.keys():
              if key not in ['multiple']:
                new_metadict[key] = old_metadict[key]
          else:
            new_metadict = old_metadict
          new_metadicts.append(new_metadict)
        self.__attr_dc_metadicts__ = copy.deepcopy(new_metadicts)
      
      ##### SYNCHRONIZE ####
      self.synchronizeObjAttrs()
      
      return message

    # ------------------------------------------------------------------------------------------
    # MetadictManager.__init_metadicts__:
    # 
    # Initializes dictionary of DC.Metadictionaries.
    # ------------------------------------------------------------------------------------------
    def __init_metadicts__(self):
      self.__attr_dc_metadicts__ = []

    # ------------------------------------------------------------------------------------------
    # MetadictManager.__get_metadicts__:
    # 
    # Returns dictionary of DC.Metadictionaries.
    # ------------------------------------------------------------------------------------------
    def get_metadicts(self): return self.__get_metadicts__()
    def __get_metadicts__(self):
      if not hasattr(self,'__attr_dc_metadicts__'):
        self.__init_metadicts__()
      return getattr(self,'__attr_dc_metadicts__')


    # ------------------------------------------------------------------------------------------
    # MetadictManager.getMetadictAttrs:
    # 
    # Returns list of attributes of DC.Metadictionaries.
    # ------------------------------------------------------------------------------------------
    def getMetadictAttrs(self, meta_type=None):
      attrs = []
      obs = self.__get_metadicts__()
      for i in range(0,len(obs)/2):
        attr = obs[i*2]
        metadictDetails = self.getMetadictDetails(attr)
        if meta_type is None or meta_type in metadictDetails.getDstMetaTypes():
          attrs.append(attr)    
      return attrs


    # ------------------------------------------------------------------------------------------
    # MetadictManager.getMetadictDetails:
    # 
    # Returns Details of DC.Metadictionary identified by attribute-name.
    # ------------------------------------------------------------------------------------------
    def getMetadictDetails(self, attr):
      obs = self.__get_metadicts__()
      ob = obs[obs.index(attr) + 1]
      return MetadictDetails(ob)


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__set_metadict__:
    # 
    # Set/add DC.Metadictionary with specified values.
    # ------------------------------------------------------------------------------------------
    def __set_metadict__(self, attr, id, name, metatype, keys='', dst_meta_types=[]):
      # Remove exisiting entry.
      obs = self.__get_metadicts__()
      if attr is not None:
        i = obs.index(attr)
        del obs[i] 
        del obs[i] 
      else: 
        i = len(obs)
      # Attribute.
      attr = getAttrName(id)
      # Values.
      values = {}
      values['id'] = id
      values['name'] = name
      values['type'] = metatype
      # datatype
      if metatype == 'multiselect':
        values['datatype'] = 'LangList'
      elif metatype == 'int':
        values['datatype'] = 'LangInt'
      elif metatype == 'date':
        values['datatype'] = 'LangDate'
      elif metatype == 'url':
        values['datatype'] = 'LangUrl'
      elif metatype == 'boolean':
        values['datatype'] = 'LangBoolean'
      elif metatype == 'image':
        values['datatype'] = 'LangImage'
      elif metatype == 'file':
        values['datatype'] = 'LangFile'
      else:
        values['datatype'] = 'LangString'
      # Keys.
      values['keys'] = []
      for key in string.split(keys,'\n'):
        key = string.strip(key)
        if len(key) > 0:
          values['keys'].append(key)
      # Meta-Types.
      if type(dst_meta_types) is not type([]):
        dst_meta_types = [dst_meta_types]
      values['dst_meta_types'] = dst_meta_types
      # Update attribute.
      obs.insert(i,values)
      obs.insert(i,attr)
      self.__attr_dc_metadicts__ = copy.deepcopy(obs)


    # ------------------------------------------------------------------------------------------
    #	MetadictManager.__move_metadict__:
    #
    #	Moves DC.Metadictionary with specified ATTR in specified DIRECTION.
    # ------------------------------------------------------------------------------------------
    def __move_metadict__(self, attr, dir):
      # Move.
      obs = self.__get_metadicts__()
      i = obs.index(attr)
      attr = obs[i] # Attribute
      values = obs[i+1] # Values
      del obs[i] 
      del obs[i] 
      # Update attribute.
      obs.insert(i+dir*2,values)
      obs.insert(i+dir*2,attr)
      self.__attr_dc_metadicts__ = copy.deepcopy(obs)


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__del_metadict__:
    # 
    # Delete DC.Metadictionary with specified ATTR.
    # ------------------------------------------------------------------------------------------
    def __del_metadict__(self, attr):
      # Delete.
      obs = self.__get_metadicts__()
      i = obs.index(attr)
      # Update attribute.
      del obs[i] # Attribute
      del obs[i] # Values
      self.__attr_dc_metadicts__ = copy.deepcopy(obs)


    ###############################################################################################
    # MetadictManager.manage_changeMetadicts:
    #
    # Werte fr DC.Metadictionaries lschen oder ndern.
    ###############################################################################################
    def manage_changeMetadicts(self, btn, lang, manage_lang, REQUEST):
        """ MetadictManager.manage_changeMetadicts """        
        
        # Delete.
        # -------
        if btn == self.getLangStr('BTN_DELETE',manage_lang):
          attr = REQUEST['attr']
          self.__del_metadict__(attr)
          deleteMetaDictAttrs(self,attr)
                    
        # Change.
        # -------
        elif btn == self.getLangStr('BTN_CHANGE',manage_lang): 
          attr = REQUEST['attr']
          # Change.
          id = string.strip(REQUEST['id'])
          name = string.strip(REQUEST['name'])
          metatype = string.strip(REQUEST['metatype'])
          keys = string.strip(REQUEST['_keys'])
          dst_meta_types = []
          for key in self.dGlobalAttrs.keys():
            if self.dGlobalAttrs[key]['meta_dict'] == 1:
              if REQUEST.has_key('_%s_dct_meta_type'%key) and \
              	 REQUEST['_%s_dct_meta_type'%key] == 1:
              	dst_meta_types.append(key)
          self.__set_metadict__(attr,id,name,metatype,keys,dst_meta_types)
          #
          new_attr = getAttrName(id)
          if attr != new_attr:
            changeMetaDictAttrs(self,attr,new_attr)
            deleteMetaDictAttrs(self,attr)
          # Display attribute..
          attr = new_attr
        
        # Move up.
        # --------
        elif btn == 'move_up':
          attr = REQUEST['attr']
          self.__move_metadict__(attr,-1)
        
        # Move down.
        # ----------
        elif btn == 'move_down':
          attr = REQUEST['attr']
          self.__move_metadict__(attr,+1)
        
        # Insert.
        # -------
        elif btn == self.getLangStr('BTN_INSERT',manage_lang): 
          id = string.strip(REQUEST['_id'])
          name = string.strip(REQUEST['_name'])
          metatype = string.strip(REQUEST['_type'])
          self.__set_metadict__(None,id,name,metatype)
          # Display attribute..
          attr = getAttrName(id)
          
        # Initialize.
        # -----------
        elif btn == self.getLangStr('BTN_INIT',manage_lang):
          initMetaDictAttrs(self)
          attr = ''
        
        ##### SYNCHRONIZE ####
        self.synchronizeObjAttrs()
        
        # Return with message.
        # --------------------
        message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
        return REQUEST.RESPONSE.redirect('manage_customizeMetadictForm?attr=%s&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(attr,lang,manage_lang,message))


# -------------------------------------------------------------------------------------------------
#  CLASS METADICTDETAILS
# -------------------------------------------------------------------------------------------------
class MetadictDetails(AccessControl.Role.RoleManager):

  # Constructor.
  # ------------
  def __init__(self, ob):
    self.setId(ob['id'])
    self.setName(ob['name'])
    self.setKeys(ob['keys'])
    self.setType(ob['type'])
    self.setDatatype(ob['datatype'])
    # Attr
    self.setAttr(getAttrName(self.getId()))
    # dst_meta_types
    if ob.has_key('dst_meta_types'):
      dst_meta_types = ob['dst_meta_types']
    else:
      dst_meta_types = ['ZMS','ZMSRubrik','ZMSDocument','ZMSFile','ZMSGraphic']
    if not type(dst_meta_types) is type([]):
      dst_meta_types = [dst_meta_types]
    self.setDstMetaTypes(dst_meta_types)
    
  def setAttr(self, attr):
    self.attr = attr

  getAttr__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getAttr(self):
    return self.attr

  def setName(self, name):
    self.name = name

  getName__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getName(self):
    return self.name

  def setKeys(self, keys):
    self.keys = keys

  getKeys__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getKeys(self):
    return self.keys

  def setId(self, id):
    self.id = id

  getId__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getId(self):
    return self.id

  def setDstMetaTypes(self, dst_meta_types):
    self.dst_meta_types = dst_meta_types

  getDstMetaTypes__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getDstMetaTypes(self):
    return self.dst_meta_types

  def setDatatype(self, datatype):
    self.datatype = datatype

  getDatatype__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getDatatype(self):
    return self.datatype
    
  def setType(self, type):
    self.type = type

  getType__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getType(self):
    return self.type

###################################################################################################
